/**
 * Required External Modules and Interfaces
 */

import express, {Request, Response} from 'express';
import {Event} from './event.interface';
import * as EventService from './events.service';
import * as iCalService from './icalgenerator.service';
import * as CredentialService from './credentials.service';
import {Guid} from 'guid-typescript';
import logger from '../../../middleware/logger';


/**
 * Router Definition
 */

export const eventsRouter = express.Router();

/**
 * Constants
 */
export const calendarNames = new Map<string, any>([
	['public', {id: 1, name: 'Nachklang_calendar'}],
	['members', {id: 2, name: 'Nachklang_internal_calendar'}],
	['management', {id: 3, name: 'Nachklang_management_calendar'}]
]);


/**
 * Controller Definitions
 */

eventsRouter.get('/:calendar/json', async (req: Request, res: Response) => {
	try {
		// Get request params
		let calendarName: string = req.params.calendar as string ?? '';
		let password: string = req.query.password as string ?? '';

		if (calendarName.length < 1) {
			res.status(401).send({'message': 'Please state the name of the calendar you want events from.'});
			return;
		}

		if (!Array.from(calendarNames.keys()).includes(calendarName)) {
			res.status(401).send({'message': 'Unknown calendar.'});
			return;
		}

		let calendarId: number = calendarNames.get(calendarName)!.id;

		if (!CredentialService.hasAccess(calendarName, password)) {
			res.status(401).send({'message': 'You do not have access to the specified calendar.'});
			return;
		}

		// Get events
		let events = await EventService.getAllEvents(calendarId);

		// Send the events back
		res.status(200).send(events);
	} catch (e: any) {
		console.log('Error handling a request: ' + e.message);
		res.status(500).send({'message': 'Internal Server Error. Try again later.'});
	}
});

eventsRouter.get('/:calendar/ical', async (req: Request, res: Response) => {
	try {
		// Get request params
		let calendarName: string = req.params.calendar as string ?? '';
		let password: string = req.query.password as string ?? '';

		if (calendarName.length < 1) {
			res.status(401).send({'message': 'Please state the name of the calendar you want events from.'});
			return;
		}

		if (!Array.from(calendarNames.keys()).includes(calendarName)) {
			res.status(401).send({'message': 'Unknown calendar.'});
			return;
		}

		let calendarId: number = calendarNames.get(calendarName)!.id;

		if (!CredentialService.hasAccess(calendarName, password)) {
			res.status(401).send({'message': 'You do not have access to the specified calendar.'});
			return;
		}

		// Get events
		let events = await EventService.getAllEvents(calendarId);

		// generate file name
		let fileName = calendarNames.get(calendarName)!.name;
		let file = await iCalService.convertToIcal(events);

		// Send the ical file back
		res.set({'Content-Disposition': 'attachment; filename=' + fileName + '.ics'});
		res.status(200).send(file);
	} catch (e: any) {
		let errorGuid = Guid.create().toString();
		logger.error('Error handling a request: ' + e.message, {reference: errorGuid});
		res.status(500).send({
			'status': 'PROCESSING_ERROR',
			'message': 'Internal Server Error. Try again later.',
			'reference': errorGuid
		});
	}
});

eventsRouter.post('/', async (req: Request, res: Response) => {
	try {
		// Get params
		let password = req.body.password;

		if (!CredentialService.checkAdminPrivileges(password)) {
			res.status(403).send({'message': 'Insufficient privileges.'});
			return;
		}

		if (
			req.body.calendarId === undefined ||
			isNullOrBlank(req.body.name) ||
			req.body.startDateTime === undefined ||
			req.body.endDateTime === undefined
		) {
			res.status(401).send({'message': 'Required parameters missing'});
			return;
		}

		let event: Event = {
			event_id: -1,
			calendar_id: req.body.calendarId,
			uuid: '',
			name: req.body.name,
			description: req.body.description ?? '',
			start_datetime: new Date(req.body.startDateTime),
			end_datetime: new Date(req.body.endDateTime),
			created_date: new Date(),
			location: req.body.location ?? '',
			created_by: req.body.createdBy ?? '',
			url: req.body.url ?? ''
		};

		let eventId = await EventService.createEvent(event);

		res.status(201).send({'message': 'Event with id ' + eventId + ' was created successfully.'});
	} catch (e: any) {
		let errorGuid = Guid.create().toString();
		logger.error('Error handling a request: ' + e.message, {reference: errorGuid});
		res.status(500).send({
			'status': 'PROCESSING_ERROR',
			'message': 'Internal Server Error. Try again later.',
			'reference': errorGuid
		});
	}
});

eventsRouter.put('/:eventId', async (req: Request, res: Response) => {
	try {
		// Get params
		let password = req.body.password;

		if (!CredentialService.checkAdminPrivileges(password)) {
			res.status(403).send({'message': 'Insufficient privileges.'});
			return;
		}

		if (
			req.params.eventId === undefined ||
			req.body.calendarId === undefined ||
			isNullOrBlank(req.body.name) ||
			req.body.startDateTime === undefined ||
			req.body.endDateTime === undefined
		) {
			res.status(401).send({'message': 'Required parameters missing'});
			return;
		}

		let event: Event = {
			event_id: parseInt(req.params.eventId, 10),
			calendar_id: req.body.calendarId,
			uuid: '',
			name: req.body.name,
			description: req.body.description ?? '',
			start_datetime: new Date(req.body.startDateTime),
			end_datetime: new Date(req.body.endDateTime),
			created_date: new Date(),
			location: req.body.location ?? '',
			created_by: req.body.createdBy ?? '',
			url: req.body.url ?? ''
		};

		let success = await EventService.updateEvent(event);


		res.status(200).send({'message': 'Event was successfully updated'});
	} catch (e: any) {
		let errorGuid = Guid.create().toString();
		logger.error('Error handling a request: ' + e.message, {reference: errorGuid});
		res.status(500).send({
			'status': 'PROCESSING_ERROR',
			'message': 'Internal Server Error. Try again later.',
			'reference': errorGuid
		});
	}
});

/**
 * Checks if a given string is null, undefined or blank
 * @param str The string to check
 */
function isNullOrBlank(str: string | null): boolean {
	return str === null || str === undefined || str.trim() === '';
}
